/**
 *  mpl3115a2.c - MPL3115A2 library for ATMega324P
 *  Datasheet: https://www.nxp.com/docs/en/data-sheet/MPL3115A2.pdf
 */

#include "mpl3115a2.h"
#include <util/delay.h>

void mpl3115a2_init(void)
{
    // TODO 3.1: Initialize MPL3115A2 with altimetry and wait for data to be ready
    twi_start();
    twi_write(MPL3115A2_ADDRESS << 1);
    twi_write(MPL3115A2_PT_DATA_CFG);
    twi_write(0x07);

    twi_start();
    twi_write(MPL3115A2_ADDRESS << 1);
    twi_write(MPL3115A2_CTRL_REG1);
    twi_write(0x39);

    twi_start();
    twi_write(MPL3115A2_ADDRESS << 1);
    twi_write(MPL3115A2_STATUS);

    twi_start();
    twi_write((MPL3115A2_ADDRESS << 1) | 0x01);

    // Wait for data ready.
    uint8_t sta = 0x00;
    while (!(sta & 0x08))
    {
        twi_read_ack(&sta);
    }
    twi_read_nack(&sta);
}

void mpl3115a2_read_pressure(uint32_t *pressure)
{
    // TODO 3.2: Read the pressure from the sensor
    twi_start();
    twi_write(MPL3115A2_ADDRESS << 1);
    twi_write(MPL3115A2_P_MSB);

    twi_start();
    twi_write((MPL3115A2_ADDRESS << 1) | 0x01);

    uint32_t msb = 0x00;
    twi_read_ack(&msb);
    uint32_t csb = 0x00;
    twi_read_ack(&csb);
    uint32_t lsb = 0x00;
    twi_read_nack(&lsb);

    // 20 bit register with 18 bits of integer part and 2 bit fractional part
    *pressure = (((msb << 12)) | ((csb << 4)) | (lsb >> 4)) >> 2;
}

void mpl3115a2_read_temperature(uint8_t *temperature)
{
    // TODO 3.2: Read the temperature from the sensor
    twi_start();
    twi_write(MPL3115A2_ADDRESS << 1);
    twi_write(MPL3115A2_T_MSB);

    twi_start();
    twi_write((MPL3115A2_ADDRESS << 1) | 0x01);

    twi_read_nack(temperature);

    twi_stop();
}
