/* twi.c - TWI library for ATMega324P */

#include "twi.h"

void twi_init(void) {
    DDRA |= (1 << PA6);
	PORTA |= (1 << PA6);

    TWCR = 0;

    TWSR &= ~(1 << TWPS0);
    TWSR &= ~(1 << TWPS1);

    TWBR = (uint8_t)TWBR_VAL;
}

void twi_start(void) {    
    TWCR = (1 << TWINT) | (1 << TWSTA) | (1 << TWEN);
    while (!(TWCR & (1 << TWINT)));
}

void twi_write(uint8_t data) {
    TWDR = data;
    TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWEA);

	while( !(TWCR & (1<<TWINT)) );
}

void twi_read_ack(uint8_t *data) {
    TWCR = (1 << TWINT) | (1 << TWEN) | (1 << TWEA);
    while( !(TWCR & (1<<TWINT)) );
    *data = TWDR;
}

void twi_read_nack(uint8_t *data) {
    TWCR = (1 << TWINT) | (1 << TWEN);
    while (!(TWCR & (1 << TWINT)));
    *data = TWDR;
}

void twi_stop(void) {
    TWCR = (1 << TWEN) | (1 << TWINT) | (1 << TWSTO);
}

void twi_discover(void) {
    for (uint8_t i = 0x00; i < 0x7F; i++)  {
        twi_start();
        twi_write((i << 1) + 1);
        if ((TWSR & 0xFB) == 0x40)
            printf("Device discovered on 0x%x\n", i);
    }
    twi_stop();
}
